classdef Pt < hgsetget
	
	%%-- Properties --%%
	
	properties (SetAccess=public)
		wl
		t
	end
	
	properties (SetAccess=private)
		h=struct('figgui',[],'axes',[],'marker',[],'etxt',struct('wl',[],'t',[]));
		opts=struct('nearpix',4,'marker',struct('marker','+','edgecolor','white','facecolor','none','size',8),'etxtformat','%4.3f','pointer','fleur');
	end
	
	properties (Dependent,SetAccess=private)
		hs
		lims
	end
	
	
	%%-- Methods --%%
	
	%% Public %%
	methods
		
		%% Constructor
		function aPt=Pt(pt,h_figGUI,varargin)
			
			%% Handle lack of arguments
			if(nargin==0)
				
				%% Warning
				warning('trplGUI:Pt:NoFigGUIHandle','No figGUI handle provided. Attempting to use callback figure');
				
				%% Try to get limits
				meas=getappdata(gcbf,'meas');
				
				%% Set coordinates top-left corner
				pt=[max(meas.wl),min(meas.t)];
				
				%% Set figGUI handle
				h_figGUI=gcbf;
				
			end
			
			%% Set figGUI handle
			if(isempty(h_figGUI) || ~ishandle(h_figGUI))
				throw(MException('trplGUI:Pt:BadFigGUIHandle','Invalid figGUI handle'));
			end
			aPt.h.figgui=h_figGUI;
				
			%% Set default axes handles
			handles=guidata(aPt.h.figgui);					% N.b. Cannot set these defaults in properties and require figGUI handles structure
			aPt.h.axes=handles.axMain;
			
			%% Parse optional parameters
			if(~isempty(varargin))
				[names,values]=varargin2pairs(varargin);
				for ind=1:length(names)
					switch(lower(names{ind}))
						case {'axes','etxtwl','etxtt'}
							aPt.setHandles(names{ind},values{ind});
						case {'nearpix', 'marker', 'markeredgecolor', 'markerfacecolor', 'markersize', 'etxtformat','pointer'};
							aPt.setOpts(names{ind},values{ind});
						otherwise
							throw(MException('trplGUI:Pt:BadArg','Invalid constructor. Expected valid handle name or option name. Received %s',names{ind}));
					end
				end
			end
			
			%% Set properties
			aPt.wl=pt(1);
			aPt.t=pt(2);
			
		end
		
		%% Destructor
		function delete(aPt)
			
			%% Clear point
			if(~isempty(aPt.h.marker) && ishandle(aPt.h.marker))
				delete(aPt.h.marker);
			end

			%% Reset edit text boxes
			if(~isempty(aPt.h.etxt.wl) && ishandle((aPt.h.etxt.wl)))
				set(aPt.h.etxt.wl,'String','wl');
			end
			if(~isempty(aPt.h.etxt.t) && ishandle((aPt.h.etxt.t)))
				set(aPt.h.etxt.t,'String','t');
			end
				
		end
		
		%% Update
		function update(aPt)
			if(~isempty(aPt.wl) && ~isempty(aPt.t))							% Catches initial property sets from constructor
				aPt.plot();
				aPt.setETxt();
			end
		end
		
		%% Proximity check
		function tf=isNear(aPt,pt)
			
			%% Get axes properties
			ax.lims=struct('wl',get(aPt.h.axes,'XLim'),'t',get(aPt.h.axes,'YLim'));
			ax.units=get(aPt.h.axes,'Units');
			set(aPt.h.axes,'Units','pixels');
			axpos=get(aPt.h.axes,'Position');
			set(aPt.h.axes,'Units',ax.units);
			ax.dims=struct('wl',axpos(3),'t',axpos(4));
			
			%% Define proximity
			near.pix=aPt.opts.nearpix;
			near.units=struct('wl',near.pix*abs(diff(ax.lims.wl))/ax.dims.wl,'t',near.pix*abs(diff(ax.lims.t))/ax.dims.t);
			
			%% Check proximity
			if(pt(1)>=(aPt.wl-near.units.wl) && pt(1)<=(aPt.wl+near.units.wl) && pt(2)>=(aPt.t-near.units.t) && pt(2)<=(aPt.t+near.units.t))
				tf=true;
			else
				tf=false;
			end
				
			
		end
		
		%% Assert coordinate value
		function [tf,bound]=assertCoord(aPt,coord,axis)
			
			%% Validate axis
			switch(lower(axis))
				case {'wl','t'}
				otherwise
					throw(MException('trplGUI:Pt:BadAxis','Axis not recognised. Expected "wl" or "t". Receieved "%s"',axis));
			end
			axis=lower(axis);
			
			%% Catch vector values
			if(~isscalar(coord))
				throw(MException('trplGUI:Pt:VecCoord','Point coordinate must be scalar. Attempted to set coordinate to a vector of length %d',length(coord)));
			end
			
			%% Catch non-numeric coordinates
			if(~isnumeric(coord))
				throw(MException('trplGUI:Pt:NonNumCoord','Point coordinate must be numeric. Attempted to set coordinate to a non-numeric value'));
			end
			
			%% Catch NaN and Inf coordinates
			if(any(~isfinite(coord)))
				throw(MException('trplGUI:Pt:NonFiniteCoord','Point coordinate must be finite. Attempted to set coordinate to NaN or Inf'));
			end
			
			%% Check coordinates within measurements limits
			if(coord<aPt.lims.(axis)(1))
				tf=false;
				bound=aPt.lims.(axis)(1);
			elseif(coord>aPt.lims.(axis)(2))
				tf=false;
				bound=aPt.lims.(axis)(2);
			else
				tf=true;
				bound=[];
			end
			
		end
		
		%% Set options
		function setOpts(aPt,varargin)
			
			%% Parse varargin
			[opts,values]=varargin2pairs(varargin);																									%#ok<PROP>
			for ind=1:length(opts)																													%#ok<PROP>
				switch(lower(opts{ind}))																											%#ok<PROP>
					case 'nearpix'
						if(isnumeric(values{ind}))
							aPt.opts.nearpix=values{ind};
						else
							warning('trplGUI:Pt:BadNearPix','Non-numeric value of NearPix option. Setting to default value (%d)',aPt.opts.nearpix);
						end
					case 'marker'
						aPt.opts.marker.marker=values{ind};
					case 'markeredgecolor'
						aPt.opts.marker.edgecolor=values{ind};
					case 'markerfacecolor'
						aPt.opts.marker.facecolor=values{ind};
					case 'markersize'
						aPt.opts.marker.size=values{ind};
					case 'etxtformat'
						aPt.opts.etxtformat=values{ind};
					case 'pointer'
						aPt.opts.pointer=values{ind};
					otherwise
						warning('trplGUI:Pt:BadOptionName','Invalid option. Expected "NearPix", "Marker", "MarkerEdgeColor", "MarkerFaceColor", "MarkerSize", "ETxtFormat". Received %s. Ignoring OptionName-OptionValue pair',opts{ind}); %#ok<PROP>
				end
			end
			
			%% Update
			aPt.update();
			
		end
		
		%% Set handles
		function setHandles(aPt,varargin)
			
			%% Parse varargin
			[names,values]=varargin2pairs(varargin);
			names=lower(names);
			for ind=1:length(names)
				switch(names{ind})
					case 'axes'
						if(isempty(values{ind}) || ishandle(values{ind}))
							aPt.h.axes=values{ind};
						else
							warning('trplGUI:Pt:BadAxesHandle','Invalid axes handle. Setting h.axes to []');
							aPt.h.axes=[];
						end
					case 'etxtwl'
						if(isempty(values{ind}) || ishandle(values{ind}))
							aPt.h.etxt.wl=values{ind};
						else
							warning('trplGUI:Pt:BadETxtWlHandle','Invalid wavelength edit text handle. Setting h.etxtwl to []');
							aPt.h.etxt.wl=[];
						end
					case 'etxtt'
						if(isempty(values{ind}) || ishandle(values{ind}))
							aPt.h.etxt.t=values{ind};
						else
							warning('trplGUI:Pt:BadETxtTHandle','Invalid time edit text handle. Setting h.etxtt to []');
							aPt.h.etxt.t=[];
						end
% 					case '_marker'
% 						if(isempty(values{ind}) || ishandle(values{ind}))
% 							aPt.h.marker=values{ind};
% 						else
% 							warning('trplGUI:Pt:BadMarkerHandle','Invalid marker handle. Setting h.marker to []');
% 							aPt.h.marker=[];
% 						end
					otherwise
						warning('trplGUI:Pt:BadHandleName','Invalid handle name. Expected "Axes", "ETxtWl" or "ETxtT". Received %s. Ignoring HandleName-HandleValue pair',names{ind});
				end
			end
			
			%% Update
			aPt.update();
			
		end

	end
	
	%% Private %%
	methods (Access=private)
		
		%% Plot point
		function plot(aPt)
			
			if(~isempty(aPt.h.axes) && ishandle(aPt.h.axes))
				
				%% Plot new point
				if(isempty(aPt.h.marker) || ~ishandle(aPt.h.marker))
					aPt.h.marker=line('Parent',aPt.h.axes,'XData',aPt.wl,'YData',aPt.t,'LineStyle','none','Marker',aPt.opts.marker.marker,'MarkerEdgeColor',aPt.opts.marker.edgecolor,'MarkerFaceColor',aPt.opts.marker.facecolor,'MarkerSize',aPt.opts.marker.size);
					iptSetPointerBehavior(aPt.h.marker,@(figHandle,currentPoint) setptr(figHandle,aPt.opts.pointer));
					iptPointerManager(get(aPt.h.axes,'Parent'),'enable');
				
				%% Modify existing point
				else
					set(aPt.h.marker,'Parent',aPt.h.axes,'XData',aPt.wl,'YData',aPt.t,'LineStyle','none','Marker',aPt.opts.marker.marker,'MarkerEdgeColor',aPt.opts.marker.edgecolor,'MarkerFaceColor',aPt.opts.marker.facecolor,'MarkerSize',aPt.opts.marker.size);
					iptSetPointerBehavior(aPt.h.marker,@(figHandle,currentPoint) setptr(figHandle,aPt.opts.pointer));
				end
				
			end
			
		end
		
		%% Set edit text boxes
		function setETxt(aPt)
			
			%% Set etxt boxes
			if(~isempty(aPt.h.etxt.wl) && ishandle(aPt.h.etxt.wl))
				set(aPt.h.etxt.wl,'String',num2str(aPt.wl,aPt.opts.etxtformat));
			end
			if(~isempty(aPt.h.etxt.t) && ishandle(aPt.h.etxt.t))
				set(aPt.h.etxt.t,'String',num2str(aPt.t,aPt.opts.etxtformat));
			end
		
		end
		
	end
	
	%% Access %%
	methods
		
		%% Set: wl
		function aPt=set.wl(aPt,wl)
			
			%% Validate value
			[ret,bound]=aPt.assertCoord(wl,'wl');
			if(~ret)
				warning('trplGUI:Pt:BadWl','Wavelength value outside measurement bound. Expected values between %g and %g. Received %g. Setting wl to bound (%g)',aPt.lims.wl(1),aPt.lims.wl(2),wl,bound); %#ok<MCSUP>
				wl=bound;
			end
			
			%% Set value
			aPt.wl=wl;
			aPt.update();
			
		end
		
		%% Set: t
		function aPt=set.t(aPt,t)
			
			%% Validate value
			[ret,bound]=aPt.assertCoord(t,'t');
			if(~ret)
				warning('trplGUI:Pt:BadT','Time value outside measurement bound. Expected values between %g and %g. Received %g. Setting t to bound (%g)',aPt.lims.t(1),aPt.lims.t(2),t,bound); %#ok<MCSUP>
				t=bound;
			end
			
			%% Set value
			aPt.t=t;
			aPt.update();
			
		end
				
		%% Get: hs
		function hs=get.hs(aPt)
			hs=[aPt.h.axes;aPt.h.etxt.wl,aPt.h.etxt.wl,aPt.h.marker];
		end
		
		%% Get: lims
		function lims=get.lims(aPt)
			meas=getappdata(aPt.h.figgui,'meas');
			lims=struct('wl',[min(meas.wl),max(meas.wl)],'t',[min(meas.t),max(meas.t)]);
		end
		
	end
	
end