%% Name:		trplLoad.m
%% Description:	Load data from Time Resolved PL measurements performed in the Chameleon lab
%% Authors:		M.J.P Alcocer [marcelo.alcocer@iit.it || marcelo.j.p.alcocer@gmail.com]
%% Licensing:	Copyright  2011  M.J.P Alcocer
%%				This file is part of tprl. trpl is free software: you can redistribute it and/or modify it under the terms of the GNU
%%				General Public License as published by the Free Software Foundation, either version 3 of the License, or any later version.
%%				trpl is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
%%				MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have
%%				received a copy of the GNU General Public License along with trpl. If not, see <http://www.gnu.org/licenses/>.

function [data,wl,t,stat,path] = trplLoad(varargin)
	
	%% Syntax:
	%%
	%%		[data,wl,t,stat] = trplLoad('path')
	%%		[data,wl,t,stat] = trplLoad('path','format')
	%%
	%% Description:
	%%
	%%		[data,wl,t,stat] = trplLoad('path') loads TRPL data from the file designated by 'path' into Matlab. Currently, two input
	%%		file formats are supported; Hamamatsu HPD-TA ITEX files (*.img), and calibrated ASCII files (*.dac). The appropriate file format
	%%		is automagically detected from the file extension.
	%%
	%%			Arguments:
	%%				'path':		Single-quoted string designating the file to be read
	%%
	%%			Returns:
	%%				data:		2D array of TRPL intensities. To match HPD-TA display, data(1,1) is the intensity at maximum wavelength and 
	%%							minimum time.
	%%				wl:			Column vector defining the horizontal wavelength axis. wl(i) corresponds to data(:,i).
	%%				t:			Column vector defining the vertical time axis. t(i) corresponds to data(i,:).
	%%				stat:		Structure containing information held in status string. The structure has the format stat.'section'.'token'.
	%%							with all values stored as strings. e.g. The number of exposures ('NrExposure' in the 'Acquisition' section)
	%%							can be found using stat.Acquisition.NrExposure
	%%
	%%		[data,wl,t,stat] = trplLoad('path','format')
	%%
	%%			Arguments:
	%%				'format':	String designating the format of the file to be read. Valid values are 'img' or 'dac'
	
	
	%% GUI path selection (if no argument)
	if(isempty(varargin))
		
		%% File selection
		exts={	'*.img','TRPL Chameleon Streak - ITEX format (*.img)'; ...
				'*.dac','TRPL Chameleon Streak - ASCII format (*.dac)'; ...
				'*.dat','TRPL L16 TCSPC - L16 binary format (*.dat)'; ...							% Is this the correct setup name?
				'*.dat','TA RegA - ASCII format (*.dat)'; ...
				'*.*','All Files (*.*)'};
		[filename,dir]=uigetfile(exts);
		if(filename==0)
			[data,wl,t,stat]=deal([]);
			return
		end
		path=[dir,filename];
	
	elseif(~ischar(varargin{1}))
		throw(MException('trpl:Load:BadPath','Path to file not recognised. Expected a string'));
	else
		path=varargin{1};
	end
	
	%% Infer file format from extension (useful when using CLI)
	[dir,filename,ext]=fileparts(path);									% TODO: Warn on multiple extensions found?
	if(~isempty(ext))
		ext=strrep(ext,'.','');
	elseif(exist([path,'.img'],'file')==2)
		ext='img';
		path=[path,'.',ext];
	elseif((exist([path,'.dac'],'file')==2))
		ext='dac';
		path=[path,'.',ext];
	elseif((exist([path,'.dat'],'file')==2))
		ext='dat';
		path=[path,'.',ext];
	end
	
	%% User specified file format
	if(nargin>1)
		if(~isempty(ext) && ~strcmp(ext,varargin{2}))
			warning('trpl:Load:MismatchedFormat','File format supplied (*.%s) does not match that of file extension (*.%s). Continuing with user supplied format',varargin{2},ext);
		end
		ext=varargin{2};
	end
	
	%% Load data according to format
	switch lower(ext)
		case 'img'
			[data,wl,t,stat]=trplLoad_IMG_TRPL(path);
		case 'dac'
			[data,wl,t,stat]=trplLoad_DAC_TRPL(path);
		case 'dat'
			if(isascii(path))
				[data,wl,t,stat]=trplLoad_DAT_RegA(path);
			else
				[data,wl,t,stat]=trplLoad_DAT_L16(path);
% 				data=sum(data,3,'native');										% Remove once have sequence handling
			end
		case ''
			throw(MException('trpl:Load:UnknownFormat','Could not infer file format from file extension. Please specify a format using the format argument'));
		otherwise
			throw(MException('trpl:Load:BadFormat','File format not yet supported. Expected *.dac or *.img formats. Received *.%s file',ext));
	end
	
	%% Assert trplGUI status section
	if(~isfield(stat,'trplGUI'))
		stat.trplGUI=struct([]);
	end
	fields={'CorrT0','CorrCurv','CorrSubRef'};
	mask=~isfield(stat.trplGUI,fields);
	if(any(mask))																				% Ensures backwards compatibility
		fields=fields(mask);
		for ind=1:length(fields)
			stat.trplGUI(1).(fields{ind})=[];
		end
	end

	%% Handle corrections done in HPD-TA
	if(isempty(stat.trplGUI.CorrCurv) && isfield(stat.Acquisition,'CurveCorr') && strcmp(stat.Acquisition.CurveCorr,'-1'))
		stat.trplGUI.CorrCurv='Unknown';
	end
	if(isempty(stat.trplGUI.CorrSubRef) && isfield(stat.Acquisition,'BacksubCorr') && strcmp(stat.Acquisition.BacksubCorr,'-1'))
		stat.trplGUI.CorrSubRef='Unknown';
	end
	
end

function [data,wl,t,stat] = trplLoad_IMG_TRPL(path)
	
	%% Open file
	fid=fopen(path,'r','l');
	if(fid<0)
		throw(MException('trpl:LoadITEX:Open','Could not open file %s',path));
	end

	%% Confirm magic number (ASCII "IM")
	mn=fread(fid,2,'uint8');
	if (~isequal(mn,uint8(['I';'M'])))
		fclose(fid);
		throw(MException('trpl:LoadITEX:MagicNumberIncorrect','Incorrect magic number for ITEX file. Expected: IM Read: %s%s',char(mn(1)),char(mn(2))));		% This is how to define an error for our own functions (in this case trplLoad_ITEX)
	end

	%% Read stat string length [bytes]
	stat_len=fread(fid,1,'uint16','l');

	%% Read image dimensions [pixels]
	dat_dim.dx=fread(fid,1,'uint16','l');
	dat_dim.dy=fread(fid,1,'uint16','l');

	%% Read offsets [pixels]
	%
	%	- When JC performed t axis is maintained at 512 whilst data is shortened
	%	- In reality, all HPDTA does is select a section of t (e.g. t(3:510))
	%	- This is defined as offsets (and in Valid Image parameter Acquisition->areSource): (X,Y,DX,DY)
	%		- DX,DY: Actual data size (e.g. 672,504)
	%		- X,Y: Offset (e.g. 0,4)
	%	- In this case, valid y scaling would be t(4+1:4+504)
	%	- c.f. with original data and scaling size given in Acquisition->areGRBScan
	%
	dat_dim.x=fread(fid,1,'uint16','l');
	dat_dim.y=fread(fid,1,'uint16','l');

	%% Read data type (HPD-TA v.9.0 User Manual Pg. 81)
	%
	%	- Whilst the camera outputs uints (e.g. Orca R2 in Chameleon lab -> uint16), data storage in ITEX files can be of a different type
	%	- This allows for negative values and processing which would otherwise lead to overflow
	%	- Only type of interest here is storage (not camera)
	%	- Camera data type should be stored somewhere though
	%	- 8bit storage is always unsigned
	%	- XITEX files extend storage data types to include floats (data_type=4 for single, datatype=5 for double)
	%
	switch fread(fid,1,'uint16','l')
		case 0
			data_type='*uint8';
		case 1			% Compressed
			fclose(fid);
			throw(MException('trpl:LoadITEX:CompressedDataType','ITEX data appears to be of an unsupported compressed type'));
		case 2
			data_type='*int16';
		case 3
			data_type='*int32';
		case 4
			data_type='*single';		% XITEX only
		case 5
			data_type='*double';		% XITEX only
		otherwise
			fclose(fid);
			throw(MException('trpl:LoadITEX:DatatypeUnknown','ITEX data appears to be of an unknown type'));
	end

	%% Skip reserved bytes
	fread(fid,50,'uint8');

	%% Read status string
	stat_str=fread(fid,stat_len,'uint8');
	stat_str=char(stat_str');

	%% Read data
	data=fread(fid,dat_dim.dx*dat_dim.dy,data_type);							% Read data all in one go (read along rows, stored in a column vector)
	data=reshape(data,dat_dim.dx,dat_dim.dy);									% Reshape data to form rows and columns
	data=data';																	% Transpose to get correct row/column format

	%% Parse stat string
	stat=ini2struct(stat_str);
	
	%% Parse scaling data
	if(isfield(stat,'Scaling') && isfield(stat.Scaling,'ScalingXType') && isfield(stat.Scaling,'ScalingYType'))
		
		%% X axis scaling
		switch uint8(str2double(stat.Scaling.ScalingXType))
			
			%% Linear scaling
			case 1
			
				%% Missing scaling value
				if(isempty(stat.Scaling.ScalingXScale))
					warning('trpl:LoadITEX:MissingXScaling','Missing X linear scaling value. Returning X axis in pixels instead');
					wl=(1:dat_dim.dx)';
					stat.Scaling.ScalingXUnit='pix';
				
				%% Build linear axis
				else
					offset.x=0;
					if(isfield(stat.Scaling,'ScalingXOffset') && ~isempty(stat.Scaling.ScalingXOffset))
						offset.x=str2double(stat.Scaling.ScalingXOffset);					% XITEX field
					end
					wl=offset.x+((0:dat_dim.dx-1)'.*str2double(stat.Scaling.ScalingXScale));
					wl=wl(dat_dim.x+1:dat_dim.x+dat_dim.dx);
				end
				
			%% Tabular scaling
			case 2
				
				%% Parse scaling address and size
				scale_x=textscan(stat.Scaling.ScalingXScalingFile,'%c%u64,%d');						% uint64 should be enough for address!
				if(strcmp(scale_x{1},'*') && isempty(scale_x{3}))
					scale_x{3}=1024;
				elseif(strcmp(scale_x{1},'+') && isempty(scale_x{3}))
					scale_x{3}=1280;
				end
				scale_x=struct('address',scale_x{2},'length',scale_x{3});
				
				%% Assert scaling address
				fi_exp_pos=ftell(fid);
				if scale_x.address~=fi_exp_pos														% Check scaling address is current file indicator position
					if scale_x.address>fi_exp_pos
						warning('trpl:LoadITEX:AdvancedXScalingPosition','X scaling data addressed later than expected in file. Expected at %d bytes. Addressed at %d bytes. All data may not have been read. Skipping to addressed position and continuing',fi_exp_pos,scale_x.address);
						fseek(fid,scale_x.address,'bof');
					end
					if scale_x.address<fi_exp_pos
						fclose(fid);
						throw(MException('trpl:LoadITEX:RetardedXScalingPosition','X scaling data addressed earlier than expected in file. Expected at %d bytes. Addressed at %d bytes. Too much data may have been read.',fi_exp_pos,scale_x.address));
					end
				end
				
				%% Read scaling data
% 				wl=fread(fid,scale_x.length,'float32','l');
				wl=fread(fid,double(scale_x.length),'float32','l');									% R2009a compatibility
				wl=wl(dat_dim.x+1:dat_dim.x+dat_dim.dx);

			otherwise
				warning('trpl:LoadITEX:BadXScalingType','Could not determine X scaling type. Expected "1" (linear) or "2" (tabular). Received "%s". Returning X axis in pixels instead',stat.Scaling.ScalingXType);
				wl=(1:dat_dim.dx)';
				stat.Scaling.ScalingXUnit='pix';
		end
		
		%% Y axis scaling
		switch uint8(str2double(stat.Scaling.ScalingYType))
			
			%% Linear scaling
			case 1

				%% Missing scaling value
				if(isempty(stat.Scaling.ScalingYScale))
					
					%% Focus mode
					if(~(strcmp(stat.Scaling.ScalingYScalingFile,'Focus mode') || isempty(stat.Scaling.ScalingYUnit)))
						warning('trpl:LoadITEX:MissingYScaling','Missing Y linear scaling value. Returning Y axis in pixels instead');
					end
					t=(1:dat_dim.dy)';
					stat.Scaling.ScalingYUnit='pix';
				
				%% Build linear axis
				else
					offset.y=0;
					if(isfield(stat.Scaling,'ScalingYOffset') && ~isempty(stat.Scaling.ScalingYOffset))
						offset.y=str2double(stat.Scaling.ScalingYOffset);					% XITEX field
					end
					t=offset.y+((0:dat_dim.dy-1)'.*str2double(stat.Scaling.ScalingYScale));
					t=t(dat_dim.y+1:dat_dim.y+dat_dim.dy);
				end
				
			%% Tabular scaling
			case 2
				
				%% Parse scaling address and size
				scale_y=textscan(stat.Scaling.ScalingYScalingFile,'%c%u64,%d');						% uint64 should be enough for address!
				if(strcmp(scale_y{1},'*') && isempty(scale_y{3}))
					scale_y{3}=1024;
				elseif(strcmp(scale_y{1},'+') && isempty(scale_y{3}))
					scale_y{3}=1280;
				end
				scale_y=struct('address',scale_y{2},'length',scale_y{3});

				%% Assert scaling address
				fi_exp_pos=ftell(fid);
				if scale_y.address~=fi_exp_pos														% Check scaling address is current file indicator position
					if scale_y.address>fi_exp_pos
						warning('trpl:LoadITEX:AdvancedYScalingPosition','Y scaling data addressed later than expected in file. Expected at %d bytes. Addressed at %d bytes. All data may not have been read. Skipping to addressed position and continuing',fi_exp_pos,scale_y.address);
						fseek(fid,scale_y(1),'bof');
					end
					if scale_y.address<fi_exp_pos
						fclose(fid);
						throw(MException('trpl:LoadITEX:RetardedYScalingPosition','Y scaling data addressed earlier than expected in file. Expected at %d bytes. Addressed at %d bytes. Too much data may have been read.',fi_exp_pos,scale_y.address));
					end
				end
				
				%% Read scaling data
% 				t=fread(fid,scale_y.length,'float32','l');
				t=fread(fid,double(scale_y.length),'float32','l');									% R2009a compatibility
				t=t(dat_dim.y+1:dat_dim.y+dat_dim.dy);
				
			otherwise
				warning('trpl:LoadITEX:BadYScalingType','Could not determine Y scaling type. Expected "1" (linear) or "2" (tabular). Received "%s". Returning Y axis in pixels instead',stat.Scaling.ScalingYType);
				t=(1:dat_dim.dy)';
				stat.Scaling.ScalingYUnit='pix';
		end
		
	else
		warning('trpl:LoadITEX:NoScaling','Could not locate scaling data. Returning axes in pixels instead');
		wl=(1:dat_dim.dx)';
		t=(1:dat_dim.dy)';
		stat.Scaling.ScalingXUnit='pix';
		stat.Scaling.ScalingYUnit='pix';
	end
	
	%% Assert axes lengths
	if(length(wl)~=dat_dim.dx)
		fclose(fid);
		throw(MException('trpl:LoadITEX:BadXAxisLength','Incompatible X axis length. X data length: %d. X axis length: %d',dat_dim.dx,length(wl)));
	end
	if(length(t)~=dat_dim.dy)
		fclose(fid);
		throw(MException('trpl:LoadITEX:BadYAxisLength','Incompatible Y axis length. Y data length: %d. Y axis length: %d',dat_dim.dy,length(t)));
	end
	
	%% Read any excess bytes
	fi_exp_pos=ftell(fid);
	[excess,excess_no]=fread(fid,inf,'uint8');
	if(excess_no~=0)
		warning('trpl:LoadITEX:ExcessBytes','Read unexpected extra %d bytes from ITEX file at %d bytes',excess_no,fi_exp_pos);
	end
	
	%% Close file
	fclose(fid);

end

function [data,wl,t,stat] = trplLoad_DAC_TRPL(path)
	
	%% Define storage data type
	datatype=struct('string','int32',...							% Default to int32 for max flexibility
					'bytes','4');
	
	%% Load PL data
	data=cast(dlmread(path,'',1,1),datatype.string);
	
	%% Load time axis
	t=dlmread(path,'',[1,0,size(data,1),0]);
	
	%% Load wavelength axis
	wl=dlmread(path,'',[0,1,0,size(data,2)]);
	wl=wl';
	
	%% Parse units
	fid=fopen(path,'r');
	units=textscan(fid,'%s',1);
	fclose(fid);
	units=regexp(units{1},'([^\|]*)(\|{1})([^\|]*)','tokens');
	units=units{1};
	units=units{1};														% This is ridiculous
	if(isempty(units{1}))
		units{1}='pix';
	end
	if(isempty(units{3}))
		units{3}='pix';
	end
	
	%% Build status structure
	
		%% Common
		units={units{3},units{1},'Counts'};
		labels={'Wavelength','Time','Intensity'};
		stat=trplLoad_DefStat(wl,t,units,labels,datatype);

end

function [data,wl,t,stat] = trplLoad_DAT_RegA(path)
	
	%% Define storage data type
	datatype=struct('string','single',...					% float32 to allow for negative deltaT values. Single should be enough.
					'bytes','4');

	%% Read TA data
	data=cast(dlmread(path,'',1,1),datatype.string);
	
	%% Read time axis
	t=dlmread(path,'',[0,1,0,size(data,2)]);
	t=t';
	
	%% Read wavelength axis
	wl=dlmread(path,'',[1,0,size(data,1),0]);
	
	%% Transpose data
	data=data';

	%% Build status structure
		
		%% Common
		units={'nm','fs','-'};
		labels={'Wavelength','Time','\DeltaT/T'};
		stat=trplLoad_DefStat(wl,t,units,labels,datatype);
		
	
end

function [data,wl,t,stat] = trplLoad_DAT_L16(path,varargin)
	
	%% Define storage data type
	datatype=struct('string','int32',...	
					'bytes','4');			% Whilst data read as uint16, store as int32 for flexibility in processing (larger range, negative values, etc)

	%% Default measurement parameters
	params=struct('bytes',struct('header',764,'subheader',0,'datatype',2),'length',struct('t',4096,'wl',16,'frame',1),'dt',0.613);	% dt in ns. uint16=2bytes
	
	%% Get file length
	listing=dir(path);
	if(isempty(listing.bytes))
		throw(MException('trpl:LoadDATL16:BadFile','Non-existant or unreadable file (%s)',path));
	end
	
	%% Calculate number of frames
	if(mod(listing.bytes-params.bytes.header,params.length.t.*params.length.wl.*params.bytes.datatype)~=0)
		throw(MException('trpl:LoadDATL16:BadFrames','Non-integer number of frames detected. Data section bytes: %d. Single frame bytes: %d (%d*%d). Remaining bytes: %d',listing.bytes-params.bytes.header,params.length.t.*params.length.wl.*params.bytes.datatype,params.length.t,params.length.wl,mod(listing.bytes-params.bytes.header,params.length.t.*params.length.wl.*params.bytes.datatype)));
	end
	params.length.frame=(listing.bytes-params.bytes.header)./(params.length.t.*params.length.wl.*params.bytes.datatype);
	
	%% Open file
	fid=fopen(path,'r','l');
	if(fid<0)
		throw(MException('trpl:LoadDATL16:Open','Could not open file %s',path));
	end
	
	%% Skip header
	fread(fid,params.bytes.header,'uint8');
	
	%% Read data
	try
		data=fread(fid,params.length.frame.*params.length.t*params.length.wl,sprintf('uint16=>%s',datatype.string));	% All frames. Assumes no subheaders
		data=reshape(data,params.length.t,params.length.wl,params.length.frame);
	catch ME
		fclose(fid);																			
		ME.rethrow;																					% Probably a memory error
	end
	
	%% Check for unread bytes
	fi_exp_pos=ftell(fid);
	[~,excess_no]=fread(fid,inf,'uint8');
	if(excess_no~=0)
		warning('trpl:LoadDATL16:ExcessBytes','Read unexpected extra %d bytes from L16 file at %d bytes',excess_no,fi_exp_pos);
	end
	
	%% Close file
	fclose(fid);
	
	%% Define time axis
	t=((0:params.length.t-1)*params.dt)';											% Start at 0 ok?
	
	%% Define wavelength axis
	wl=(1:params.length.wl)';														% Add wavelength axis and correction here	
	
	%% Build status structure
	
		%% Common
		units={'pix','ns','Counts'};
		labels={'Wavelength','Time','Frequency'};
		stat=trplLoad_DefStat(wl,t,units,labels,datatype);
	

end

function stat = trplLoad_DefStat(wl,t,units,labels,datatype)
% Create common fields of default status structure
%
%	- Used to build default (i.e. essiential for trplSave) status structure
%	- Used only when loading non-ITEX data where no/limited status structure is present (i.e. in all load subfunctions except trplLoad_IMG_TRPL)
%	- Extends ITEX (XITEX) by adding to existing INI sections (e.g. Scaling->ScalingXOffset) => incompatible with HPD-TA
%	- Shouldn't be a problem though as shouldn't need to call this on TRPL data to be subsequently analysed with HPD-TA
%
%	- XITEX status structure modifications/extensions:
%		- Scaling.Scaling*Offset
%			- Data:			Number
%			- Descritpion:	Offset for linear scaling allowing linearly scaled axes with non-zero start (why this was not included in the original ITEX
%							definition is beyond me...)
%		- Acquisition.DataTypeString
%			- Data:			Storage data type (uint8, int16, int32, single, double)
%			- Description:	String representation of data type to be used for XITEX storage. Allows comparison against Matlab class for assertion.
%
%%

	%% X Scaling
	if(islinspace(wl))
		%% Linear
		stat.Scaling.ScalingXType='1';									% Type 1: Linear scaling
		stat.Scaling.ScalingXScale=sprintf('%f',wl(2)-wl(1));
		stat.Scaling.ScalingXOffset=sprintf('%f',wl(1));				% XITEX field - Used for linear scaling with offset
		stat.Scaling.ScalingXScalingFile='';
	else
		%% Tabluar
		stat.Scaling.ScalingXType='2';									% Type 2: Tabular scaling
		stat.Scaling.ScalingXScale='1';									% Arbitrary value (following HPD-TA style)
		stat.Scaling.ScalingXOffset='';
		stat.Scaling.ScalingXScalingFile='#0000000,0000';				% Zero pad to facilitate indicator calculations
	end
	stat.Scaling.ScalingXUnit=units{1};
	
	%% Y Scaling
	if(islinspace(t))
		%% Linear
		stat.Scaling.ScalingYType='1';
		stat.Scaling.ScalingYScale=sprintf('%f',t(2)-t(1));
		stat.Scaling.ScalingYOffset=sprintf('%f',t(1));
		stat.Scaling.ScalingYScalingFile='';
	else
		%% Tabluar
		stat.Scaling.ScalingYType='2';
		stat.Scaling.ScalingYScale='1';
		stat.Scaling.ScalingYOffset='';
		stat.Scaling.ScalingYScalingFile='#0000000,0000';
	end
	stat.Scaling.ScalingYUnit=units{2};
	
	%% Acquisition
	stat.Acquisition=struct('ZAxisLabel',labels{3},...
							'ZAxisUnit',units{3},...
							'DataTypeString',datatype.string,...		% XITEX field - Used for determination of storage datatype
							'BytesPerPixel',datatype.bytes);	

end
