%% Name:		trplPlot.m
%% Description:	Plot data from Time Resolved PL data obtained in the Chameleon lab
%% Authors:		M.J.P Alcocer [marcelo.alcocer@iit.it  || marcelo.j.p.alcocer@gmail.com]
%% Licensing:	Copyright  2011  M.J.P Alcocer
%%				This file is part of tprl. trpl is free software: you can redistribute it and/or modify it under the terms of the GNU
%%				General Public License as published by the Free Software Foundation, either version 3 of the License, or any later version.
%%				trpl is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
%%				MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have
%%				received a copy of the GNU General Public License along with trpl. If not, see <http://www.gnu.org/licenses/>.

function h = trplPlot(varargin)
	
	%% Syntax:
	%%
	%%		h = trplPlot(data,wl,t)
	%%
	%% Description:
	%%
	%%		h = trplPlot(data,wl,t) plots Chameleon TRPL data in the current (or new) figure using the same layout as HPD-TA. This is
	%%		achieved using a linearly interpolated pseudo-color plot (pcolor).
	%%
	%%			Arguments:
	%%				data:		2D array of TRPL intensities in the same format as returned by trplLoad.
	%%				wl:			Column vector defining the horizontal wavelength axis with wl(i) corresponding to data(:,i).
	%%				t:			Column vector defining the vertical time axis with t(i) corresponding to data(i,:).
	%%
	%%			Returns:
	%%				h:			Handle to the surface object
	%%
	%%		h = trplPlot(axes_handle,data,wl,t) plots into the axes with handle axes_handle instead of the current axes.
	%%
	
	%% Parse varargin
	h_ax=[];
	switch(nargin)
		case 3
			[data,wl,t]=varargin{:};
		case 4
			[h_ax,data,wl,t]=varargin{:};
		otherwise
			throw(MException('trpl:Plot:NArgs','Invalid number of arguments passed to trplPlot. Expected 3 or 4 arguments. Received %d arguments',nargin));
	end
	
	%% Plot using interpolated pseudo-color plot
	if((~isempty(h_ax)) && ishandle(h_ax))
		h=pcolor(h_ax,wl,t,double(data));					% Data to double as required for cdata
	else
		h=pcolor(wl,t,double(data));
		h_ax=get(h,'Parent');
	end
	
	%% Cosmetic tidy-up
	set(h,'FaceColor','interp','LineStyle','none');
	set(h_ax,'XDir','reverse','YDir','reverse');
	
end