%% Name:		trplReconv.m
%% Description:	Reconvolution of Time Resolved PL dynamics
%% Authors:		M.J.P Alcocer [marcelo.alcocer@iit.it || marcelo.j.p.alcocer@gmail.com]
%% Licensing:	Copyright  2014  M.J.P Alcocer
%%				This file is part of tprl. trpl is free software: you can redistribute it and/or modify it under the terms of the GNU
%%				General Public License as published by the Free Software Foundation, either version 3 of the License, or any later version.
%%				trpl is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
%%				MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have
%%				received a copy of the GNU General Public License along with trpl. If not, see <http://www.gnu.org/licenses/>.

function out = trplReconv(t,h_in,irf,varargin)

	%% Syntax:
	%%
	%%		cvkin = trplReconv(t,kin,irf)
	%%		cvkin = trplReconv(t,kin,irf,'OptionName',OptionValue)
	%%
	%% Description:
	%%
	%%		cvkin = trplReconv(t,kin,irf) numerically reconvolves the analytical kinetic kin with the instrument response function irf. Both
	%%		signals are defined within the temporal range described by the temporal axis t. The returned convolved kinetic cvkin is cropped,
	%%		temporally translated and scaled to match the position and amplitude of the analytical kinetic.
	%%
	%%			Arguments:
	%%				t:		Vector defining the time axis for the signals.
	%%				kin:	Function decribing the analytical kinetic. This must be a function handle to a function which accepts a single
	%%						vector argument t, and returns a vector describing the analytical kinetic over this temporal range. Extra
	%%						parameters can be passed to the function by parameterising (e.g. using anonymous functions, nested functions or
	%%						global variables). Please refer to the Optimization Toolbox documentation on 'Passing Extra Parameters' for
	%%						further details. The analytical kinetic must exhibit a maximum within the temporal range.
	%%				irf:	Vector or function describing the instrument response. As a function, irf must be a function handle to a function
	%%						which accepts a single vector argument t, and returns a vector describing the instrument response over this
	%%						temporal range. Care should be taken to ensure that the instrument response is fully defined over the temporal
	%%						range and is not truncated.
	%%
	%%			Returns:
	%%				cvkin:	The convolved kinetic as a vector. cvkin is cropped, temporally translated and scaled so as to match positions
	%%						and amplitudes of first maxima with kin when defined over the temporal axis t.
	%%
	%%
	%%		cvkin = trplReconv(t,kin,irf,'OptionName',OptionValue) sets named option 'OptionName' to the value OptionValue. Options modify
	%%		the behaviour of trplReconv. Valid option names/values are:
	%%
	%%			'OptionName'		OptionValue	{default}			Description
	%%
	%%			'FitMode'			true | {false}					Return zeros(size(t)) on error and supress error messages
	%%			'MinPeakFraction'	numeric {0.1}					Minimum height for finding of kinetic maxima. This is expressed a
	%%																fraction of the maximum value of the kinetic.
	%%
	%% Examples:
	%%
	%%		Reconvolution of difference-exponential kinetic with a bi-gaussian IRF described as a function:
	%%
	%%			% Define temporal axis
	%%			t=linspace(-20,140,672);
	%%
	%%			% Define analytical functions for kinetic and IRF
	%%			h_expd2=@(t,A,tau) (t>=0).*((A(1).*exp(-t./tau(1)))-(A(2).*exp(-t./tau(2))));
	%%			h_gauss2=@(t,A,t0,sigma) ...
	%%						(A(1).*exp(-(((t-t0(1))./(sqrt(2).*sigma(1))).^2)))+(A(2).*exp(-(((t-t0(2))./(sqrt(2).*sigma(2))).^2)));
	%%
	%%			% Parameterize analytical functions using anonymous functions
	%%			A.in=[1,1];tau=[50,10];
	%%			h_in=@(x) h_expd2(x,A.in,tau);
	%%			A.irf=[0.75,0.5]; t0=[30,70]; sigma=[7,5];
	%%			h_irf=@(x) h_gauss2(x,A.irf,t0,sigma);
	%%
	%%			% Perform reconvolution
	%%			cvkin=trplReconv(t,h_in,h_irf);
	%%			plot(t,[h_irf(t);h_in(t);cvkin]);
	%%
	%%		Reconvolution of same analytical kinetic with the same IRF now described as a vector:
	%%
	%%			irf=h_irf(t);
	%%			cvkin=trplReconv(t,h_in,irf);
	%%			plot(t,[irf;h_in(t);cvkin]);
	%%		
	%%		Usage within cftool
	%%
	%%			% Define a custom fit function in a file
	%%			function pl=myFit(t,A,tau,sigma)
	%%				h_exp1=@(t,A,tau) (t>=0).*(A.*exp(-t./tau));
	%%				h_gauss1=@(t,A,t0,sigma) A.*exp(-(((t-t0)./(sqrt(2).*sigma)).^2));
	%%				A_irf=1;																% IRF amplitude is arbitrary
	%%				t0=(max(t)-min(t))/2;													% Middle of temporal axis
	%%				h_in=@(x) h_exp1(x,A,tau);
	%%				h_irf=@(x) h_gauss1(x,A_irf,t0,sigma);
	%%				pl=trplReconv(t,h_in,h_irf,'FitMode',true);
	%%			end
	%%
	%%			% Call fit function in cftool via the custom equation dialog (may take a few seconds to register correctly)
	%%			myFit(x,A,tau,sigma)
	%%


	
	%% Parse varargin
	[varg.names,varg.values]=varargin2pairs(varargin);
	opts=struct('minpeakfraction',0.1,'fitmode',false);						% Default options
	opt_funcs=struct(	'isdim',@(x,dim) all(size(x)==dim), ...				% Conditional functions for option validation
						'frac',@(x) (all(x<=1) && all(x>0)) ...
					);
	for ind_opts=1:(length(varg.names))
		switch(lower(varg.names{ind_opts}))
			case 'minpeakfraction'
				if(~setOption('minpeakfraction',{@(x) opt_funcs.isdim(x,[1,1]),@isnumeric, opt_funcs.frac}))
					warning('trpl:Reconv:OptMinPkFrac','Option "MinPeakFraction" value not a scalar fraction. Expected value in interval ]0,1]. Received: %g. Continuing with default option value (%g)',varg.values{ind_opts}(1),opts.minpeakfraction);
				end
			case 'fitmode'
				if(~setOption('fitmode',{@(x) opt_funcs.isdim(x,[1,1]),@islogical}))
					warning('trpl:Reconv:OptFitMode','Option "FitMode" value not a scalar logical. Expected: true|false. Received: %d. Continuing with default option value (%d)',varg.values{ind_opts}(1),opts.fitmode);
				end
			otherwise
				throw(MException('trpl:Reconv:BadOpt','Option not recognised: %s',varg.names{ind_opts}));
		end
	end
	
	%% Handle temporal axis dimensions
	t=struct('exp',t);
	if(size(t.exp,2)<size(t.exp,1))
		flag_col=true;
		t.exp=t.exp';
	else
		flag_col=false;
	end
	if(~trplReconv_Assert(length(t.exp)>2,MException('trpl:Reconv:ShortAxis','Time vector too short. Minimum length: 3. Actual length: %d',length(t.exp))))		% Avoids errors with findpeaks
		return
	end
		
	%% Calculate temporal axis parameters
	n.exp=length(t.exp);
	t.lin=linspace(t.exp(1),t.exp(end),n.exp);
	t.step=t.lin(2)-t.lin(1);
	t.intercept=t.lin(1)-t.step;
	
	%% Generate IRF (if function)
	if(isa(irf,'function_handle'))
		irf=irf(t.lin);
	end

	%% Generate preliminary input signal
	in=h_in(t.lin);
	
	%% Validate input signal maximum
	[~,ind.in_max]=findpeaks(in,'NPeaks',1,'MinPeakHeight',opts.minpeakfraction*max(in));
	if(~trplReconv_Assert(~isempty(ind.in_max),MException('trpl:Reconv:InPeaks','No peaks found in input signal over given temporal range. Cannot continue with reconvolution')))
		return						% No peaks OR max at end (growth)
	end
	
	%% Preliminary convolution
	out=conv(in,irf);
	
	%% Calculate signal extension
	[~,ind.out_max]=findpeaks(out,'NPeaks',1,'MinPeakHeight',opts.minpeakfraction*max(out));
	if(~trplReconv_Assert(~isempty(ind.out_max),MException('trpl:Reconv:OutPeaks','No peaks found in output signal over given temporal range. Cannot continue with reconvolution')))
		return						% No peaks OR max at end (growth)
	end
	n.lin=ind.out_max-ind.in_max+length(t.exp)-1;
	
	%% Extend signal
	t.lin=t.lin(1):t.step:(t.step*n.lin)+(t.lin(1)-t.step);
	in=h_in(t.lin);
	
	%% Final convolution
	out=conv(in,irf);
	
	%% Crop and normalise (to input signal peak amplitude)
	ind.out=[(ind.out_max-ind.in_max+1),ind.out_max-ind.in_max+n.exp];
	out=out(ind.out(1):ind.out(2));
	out=(out.*in(ind.in_max))/out(ind.in_max);

	%% Interpolate to original temporal axis
	out=interp1(t.lin(1:n.exp),out,t.exp);
	if(opts.fitmode)
		out(isnan(out))=0;							% Replace NaNs with 0s for fitting
	end

	%% Match temporal axis dimension
	if(flag_col)
		out=out';
	end
	
	
	
	
	%%-- Inline helper functions --%%

	function tf=setOption(opt,conds)
		mask=false(1,length(conds));
		for ind_conds=1:length(conds)
			mask(ind_conds)=conds{ind_conds}(varg.values{ind_opts});
		end
		if(all(mask))
			opts.(opt)=varg.values{ind_opts};
			tf=true;
		else
			tf=false;
		end
	end

	function tf=trplReconv_Assert(cond,ME)
		tf=true;
		if(~cond)
			tf=false;
			if(opts.fitmode)
				out=zeros(size(t.exp));
				if(flag_col)
					out=out';
				end
			else
				ME.throwAsCaller;
			end
		end
				
	end
	
end
