%% Name:		trplSpecTool.m
%% Description:	Tool for manipulating spectra of Time Resolved PL data obtained in the Chameleon lab
%% Authors:		M.J.P Alcocer [marcelo.alcocer@iit.it || marcelo.j.p.alcocer@gmail.com]
%% Licensing:	Copyright  2011  M.J.P Alcocer
%%				This file is part of tprl. trpl is free software: you can redistribute it and/or modify it under the terms of the GNU
%%				General Public License as published by the Free Software Foundation, either version 3 of the License, or any later version.
%%				trpl is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
%%				MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have
%%				received a copy of the GNU General Public License along with trpl. If not, see <http://www.gnu.org/licenses/>.

function outstruct=trplSpecTool(file_name,times,varargin)

	%% Syntax:
	%%
	%%		outstruct = trplSpecTool('filename',times)
	%%		outstruct = trplSpecTool('filename',times,'OptionName',OptionValue)
	%%
	%% Description:
	%%
	%%		outstruct = trplSpecTool('filename',times) opens the Chameleon TRPL data contained in 'filename' and extracts integrated spectra
	%%		gated between the time intervals defined by times.
	%%
	%%			Arguments:
	%%				'filename':	A single filename or a cell array of multiple filenames, as either absolute or relative path strings. If
	%%							'filename' is an empty array (i.e. []), a dialog is displayed to choose files from the filesystem.
	%%				times:		Time intervals over which to get integrated spectra. A single set of intervals (i.e. one n*2 array) or multiple
	%%							sets of intervals (i.e. a cell array with m nm*2 arrays) are accepted. If a single set of intervals is given,
	%%							it is used for all files in the	'filename' argument. If multiple sets of intervals are given, the interval set
	%%							times{i} will be used for filename{i}, and thus the dimensions of times and 'filename' must agree.
	%%
	%%			Returns:
	%%				outstruct:	A structure containing the gated spectra. outstruct will contain a structure for each file in 'filenames'
	%%							with the sub-fields wl and pl containing the wavelength axis and integrated spectra respectively.
	%%
	%%
	%%		outstruct = trplSpecTool('filename',times,'OptionName',OptionValue) sets named option 'OptionName' to the value OptionValue. Options
	%%		modify the operation of trplGateSpec. Valid option names/values are:
	%%
	%%			'OptionName'		OptionValue	{default}			Description
	%%
	%%			'Normalize'			true | {false]					Normalize the spectra to unity at maximum
	%%			'Scaling'			numeric array {ones}			Scaling coefficients for spectra such that scaling(i) acts on filename{i}
	%%			'ToWorkspace'		true | {false}					Export the gated spectra to the base workspace as arrays
	%%			'ToCFTool'			true | {false}					Export the gated spectra to the MATLAB curve-fitting tool
	%%			'Plot'				{true} | false					Plots the gated spectra
	%%			'Copy'				true | {false}					Copy the gated spectra to clipboard
	%%	
	

	%% Parse varargin %%
	[varg.names,varg.values]=varargin2pairs(varargin);
	opts=struct('scaling',[],'normalize',false,'toworkspace',false,'tocftool',false,'plot',true,'copy',false);				% Default options
	for ind_opts=1:(length(varg.names))
		switch(lower(varg.names{ind_opts}))
			case 'scaling'
				if(~setOptionNumArray('scaling'))
					warning('trpl:SpecTool:OptScaling','Option "Scaling" value not numeric array. Continuing with default option value');
				end
			case 'toworkspace'
				if(~setOptionLogical('toworkspace'))
					warning('trpl:SpecTool:OptWksp','Option "ToWorkspace" value not a scalar logical. Expected: true|false. Received: %d. Continuing with default option value (%d)',varg.values{ind_opts},opts.toworkspace);
				end
			case 'tocftool'
				if(~setOptionLogical('tocftool'))
					warning('trpl:SpecTool:OptCFTool','Option "ToCFTool" value not a scalar logical. Expected: true|false. Received: %d. Continuing with default option value (%d)',varg.values{ind_opts},opts.tocftool);
				end
			case {'normalize','normalise'}
				if(~setOptionLogical('normalize'))
					warning('trpl:SpecTool:OptNorm','Option "Normalize" value not a scalar logical. Expected: true|false. Received: %d. Continuing with default option value (%d)',varg.values{ind_opts},opts.normalize);
				end
			case 'plot'
				if(~setOptionLogical('plot'))
					warning('trpl:SpecTool:OptPlot','Option "Plot" value not a scalar logical. Expected: true|false. Received: %d. Continuing with default option value (%d)',varg.values{ind_opts},opts.plot);
				end
			case 'copy'
				if(~setOptionLogical('copy'))
					warning('trpl:SpecTool:OptCopy','Option "Copy" value not a scalar logical. Expected: true|false. Received: %d. Continuing with default option value (%d)',varg.values{ind_opts},opts.copy);
				end
			otherwise
				throw(MException('trpl:SpecTool:BadOpt','Option not recognised: %s',varg.names{ind_opts}));
		end
	end

	%% Get filenames (if none given) %%
	file_dir=[];
	if(isempty(file_name))
		exts={'*.img','ITEX image (*.img)';'*.dac','Calibrated ASCII image (*.dac)';'*.*','All Files (*.*)'};
		[file_name,file_dir]=uigetfile(exts,'Multiselect','on');
		if(isscalar(file_name) && file_name==0)
			return
		end
	end
	
	%% Encapsulate single path
	if(~iscell(file_name))
		file_name={file_name};
	end	
	no_files=length(file_name);
	
	%% Replicate and encapsulate single time interval set
	if(~iscell(times))
		c_times=cell(1,no_files);
		c_times(:)={times};
		times=c_times;
		clear('c_times');
	end
	
	%% Pre-loop checks
	if(length(times)~=no_files)
		throw(MException('trpl:SpecTool:IntervalDims','Inconsistent time intervals dimensions. Number of time intervals must equal number of files. Number of files: %d. Number of time intervals: %d',no_files,length(times)));
	end
	if(~isempty(opts.scaling) && length(opts.scaling)~=no_files)
		throw(MException('trpl:SpecTool:ScalingDims','Inconsistent scaling coefficient dimensions. Number of scaling coefficients must equal number of files. Number of files: %d. Number of scaling coefficients: %d',no_files,length(opts.scaling)));
	end
	
	%% Loop though files
	for ind_file=1:no_files
		
		%% Parse file parts
		path=[file_dir,file_name{ind_file}];
		[dir,name]=fileparts(path);
		
		%% Load data
		[data,wl,t,stat]=trplLoad(path);
		
		%% Get gated spectra
		[specs,inds]=trplGetSpec(data,t,times{ind_file});

		%% Check for mixed spectra
		single_mask=(diff(inds')'==0);
		if(~(all(single_mask) || all(~single_mask)))
			warning('trpl:SpecTool:MixedSpecs','Both single-time and integrated spectra . Relative scaling between them may be incorrect');
		end
		
		%% Normalise to interval width and scaling coefficient
		scale=abs(diff(t(inds)')');
		scale(single_mask)=1;
		if(~isempty(opts.scaling))
			scale=scale/(opts.scaling(ind_file));
		end
		specs=bsxfun(@rdivide,specs,scale);
		
		%% Normalise spectra to max
		if(opts.normalize)
			specs=bsxfun(@rdivide,specs,max(specs,[],2));
		end
		
		%% Handle plot option
		if(opts.plot)
			
			%% Plot spectra
			figure('NumberTitle','off','Name','trplSpecTool - Plot','color',get(0,'defaultUicontrolBackgroundColor'),'ToolBar','figure');
			plot(wl,specs);
		
			%% Build legend
			no_gates=size(inds,1);
			leg=cell(no_gates,1);
			for ind_time=1:no_gates
				leg{ind_time}=sprintf('%g%g %s',times{ind_file}(ind_time,:),stat.Scaling.ScalingYUnit);				% Slightly incorrect display, but ok
			end
			legend(leg);

			%% Add axis title
			title(name,'Interpreter','none','FontWeight','bold');
			
		end
		
		%% Handle copy option
		if(opts.copy)
			
			%% Preallocate
			if(ind_file==1)
				no_ints=size(vertcat(times{:}),1);
				cp_header=cell(2,no_files);
				cp_data=zeros(size(specs,2),no_ints+no_files);
				cp_ptr=0;
			end
			
			%% Assign
			no_ints=size(times{ind_file},1);
			cp_header{1,ind_file}=[name,ones(1,no_ints+1)*9];								% 9 == \t
			cp_header{2,ind_file}=['wl',9,sprintf('%g-%g ps\t',times{ind_file}')];
			cp_data(:,cp_ptr+1)=wl;
			cp_data(:,cp_ptr+1+1:cp_ptr+1+no_ints)=specs';
			cp_ptr=cp_ptr+1+no_ints;
			
		end
		
		%% Loop through intervals
		if(nargout>0 || opts.toworkspace || opts.tocftool)
			for ind_int=1:size(times{ind_file},1)

				%% Handle toworkspace option
				if(opts.toworkspace)

					%% Generate default variable name
					ws_var=genvarname(name);								% N.b. This just removes forbidden names. export2wsdlg handles naming conflicts.
					ws_var=genvarname(sprintf('%s_t%.3g_%.3g',ws_var,times{ind_file}(ind_int,:)));

					%% Open export to workspace dialog
					h_ex2ws.fig=export2wsdlg({'Wavelength:','PL Intensity:'},{[ws_var,'_wl'],[ws_var,'_pl']},{wl,specs(ind_int,:)'},'Export gated spectra to workspace');

					%% Minor cosmetic surgery
					h_ex2ws.pbut_OK=findobj(get(h_ex2ws.fig,'Children'),'flat','String','OK');
					set(h_ex2ws.pbut_OK,'FontWeight','normal');
					uicontrol(h_ex2ws.pbut_OK);
					h_ex2ws.etxt=findobj(get(h_ex2ws.fig,'Children'),'flat','Style','edit');
					set(findjobj(h_ex2ws.etxt),'FocusGainedCallback',@trplFitPrep_etxt_FocusGainedCallback);
					uiwait(h_ex2ws.fig);

				end

				%% Handle tocftool option
				if(opts.tocftool)
					var_wl=genvarname(sprintf('%s_t%.3g_%.3g_wl',name,times{ind_file}(ind_int,:)),evalin('base','who'));
					var_spec=genvarname(sprintf('%s_t%.3g_%.3g_pl',name,times{ind_file}(ind_int,:)),evalin('base','who'));
					assignin('base',var_wl,wl);											% Workaround to limitation of dynamic variables with nested functions
					assignin('base',var_spec,specs(ind_int,:)');
					evalin('base',sprintf('cftool(%s,%s);',var_wl,var_spec));
					evalin('base',sprintf('clear %s %s;',var_wl,var_spec));
				end

				%% Build return structure
				if(nargout>0)
					outstruct.(genvarname(name,who)).(genvarname(sprintf('t%.3g_%.3g',times{ind_file}(ind_int,:))))=struct('wl',wl,'pl',specs(ind_int,:)');
				end

			end
		end
		
	end
	
	%% Copy to clipboard
	if(opts.copy)
		form_str=repmat('%g\t',1,size(cp_data,2));
		form_str(end)='n';
		cp_header{1,end}(end)=10;											% 10 == \n
		cp_header{2,end}(end)=10;
		cp_header=cp_header';
		clipboard('copy',[cp_header{:},sprintf(form_str,cp_data')]);
	end
	
	
	
	
	
	
	%%-- Inline helper functions --%%
	
	%% Set logical option %%
	function ret=setOptionLogical(opt)
		if(isscalar(varg.values{ind_opts}) && islogical(varg.values{ind_opts}))
			opts.(opt)=varg.values{ind_opts};
			ret=true;
		else
			ret=false;
		end
	end

	%% Set numerical option %%
	function ret=setOptionNumArray(opt)
		if(isnumeric(varg.values{ind_opts}))
			opts.(opt)=varg.values{ind_opts};
			ret=true;
		else
			ret=false;
		end
	end
	

end

%% Edit text focus callback %%
function trplFitPrep_etxt_FocusGainedCallback(jhObject, jeventdata)

	set(jhObject,'selectionStart',0);
	set(jhObject,'selectionEnd',length(get(jhObject,'text')));
	
end
