%% Name:		trplT0Find.m
%% Description:	Find photoluminscence rising-edge of Time Resolved PL data obtained in the Chameleon lab
%% Authors:		M.J.P Alcocer [marcelo.alcocer@iit.it || marcelo.j.p.alcocer@gmail.com]
%% Licensing:	Copyright  2011  M.J.P Alcocer
%%				This file is part of tprl. trpl is free software: you can redistribute it and/or modify it under the terms of the GNU
%%				General Public License as published by the Free Software Foundation, either version 3 of the License, or any later version.
%%				trpl is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
%%				MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have
%%				received a copy of the GNU General Public License along with trpl. If not, see <http://www.gnu.org/licenses/>.

function [t0,ind_t0,ind_tsig] = trplT0Find(kin,t,varargin)
	
	%% Syntax:
	%%
	%%		[t0,ind_t0,ind_tsig] = trplT0Find(kin,t)
	%%		[t0,ind_t0,ind_tsig] = trplT0Find(kin,t,'OptionName',OptionValue)
	%%
	%% Description:
	%%
	%%		[t0,ind_t0,ind_tsig] = trplT0Find(kin,t) finds the photoluminscence rising-edge in the Chameleon TRPL kinetic profile described by
	%%		kin and t and returns its time, t0, and temporal axis index, ind_t0. This is achieved by an adaptive local regression filtering 
	%%		algorithm, which smooths the kinetic before searching for the maximum value. As such, trplT0Find will not work well on growing
	%%		kinetics. The pre-rising edge index, ind_tsig is also returned as defined by the 3 sigma point of a gaussian distrubition.
	%%
	%%			Arguments:
	%%				kin:	1D array of TRPL intensities in the same format as returned by trplLoad.
	%%				t:		Column vector defining the vertical time axis with t(i) corresponding to kin(i).
	%%
	%%			Returns:
	%%				t0:			The time at which the rising-edge occurs.
	%%				ind_t0:		The time-axis index of t0, such that t0=t(ind_t0).
	%%				ind_tsig:	The time-axis index of tsig, such that tsig=t(ind_tsig).
	%%
	%%
	%%		[t0,ind] = trplT0Find(kin,t,'OptionName',OptionValue) sets named option 'OptionName' to the value OptionValue. Options
	%%		modify the operations which trplT0Find performs to the kinetics. Valid option names/values are:
	%%
	%%			'OptionName'		OptionValue	{default}			Description
	%%
	%%			'draw'				{true} | false					Plot output
	%%			'MaxFraction'		numeric {1}						Fractional value of the maximum intensity at to set as t=0 
	%%			'StDevFraction'		numeric {auto}					Fractional value of the input kinetic standard-deviation for use in 
	%%																adaptive local regression filtering for finding rising-edge
	%%			'Sigma'				numeric {3}						Gaussian standard-deviation for identification of pre-rising edge region
	%%
	
	%% Parse varargin
	[varg.names,varg.values]=varargin2pairs(varargin);
	opts=struct('draw',true,'stdevfraction',[],'maxfraction',1,'sigma',3);		% Default options
	for ind_opts=1:(length(varg.names))
		switch(lower(varg.names{ind_opts}))
			case 'draw'
				if(~setOptionLogical('draw'))
					warning('trpl:T0Find:OptDraw','Option "Draw" value not a scalar logical. Expected: true|false. Received: %d. Continuing with default option value (%d)',varg.values{ind_opts},opts.draw);
				end
			case 'stdevfraction'
				if(~setOptionFloat('stdevfraction',[0,1]))
					warning('trpl:T0Find:OptStDev','Option "StDevFraction" value not a scalar fraction. Expected value in interval ]0,1]. Received: %g. Continuing with default option value (%s)',varg.values{ind_opts},'auto');
				end
			case 'maxfraction'
				if(~setOptionFloat('maxfraction',[0,1]))
					warning('trpl:T0Find:OptMaxFrac','Option "MaxFraction" value not a scalar fraction. Expected value in interval ]0,1]. Received: %g. Continuing with default option value (%g)',varg.values{ind_opts},opts.maxfraction);
				end
			case 'sigma'
				if(~setOptionFloat('sigma',[0,10]))
					warning('trpl:T0Find:OptSigma','Option "Sigma" value not a positive scalar. Expected value in interval ]0,10] Received: %g. Continuing with default option value (%g)',varg.values{ind_opts},opts.sigma);
				end
			otherwise
				throw(MException('trpl:T0Find:BadOpt','Option not recognised: %s',varg.names{ind_opts}));
		end
	end
		
	%% Get information on kinetics %%
	kins.raw.pl=kin;
	kins.length=size(kins.raw.pl,1);
	kins.raw.max.value=max(kins.raw.pl);
	kins.raw.pl=kins.raw.pl./(kins.raw.max.value);
	kins.raw.sd=std(diff(kins.raw.pl));
	
	%% Set default standard devation fraction
	if(isempty(opts.stdevfraction))
		opts.stdevfraction=0.005*(kins.raw.sd^-1.2);					%% Empirical power law fit for optimal sd_frac value depending on S/N
		if(opts.stdevfraction>1 || isnan(opts.stdevfraction))
			opts.stdevfraction=1;
		end
	end
	
	%% Perform adaptive local regression filtering %%
	kins.smooth.sd=kins.raw.sd;
	kins.smooth.pl=kins.raw.pl;
	span=0;
	while(kins.smooth.sd>(opts.stdevfraction*kins.raw.sd))
		span=span+4;
		if(span>=kins.length)
			throw(MException('trpl:T0Find:MaxSpan','Span exceeded kinetic length. Could not continue with smoothing. Span: %d. Kinetic Length: %d',span,kins.length));
		end
		kins.smooth.pl=smooth(kins.raw.pl,span,'lowess');
		if(isempty(kins.smooth.pl))
			throw(MException('trpl:T0Find:Rough','Smoothing failed. Reason unknown.'));
		end
		kins.smooth.sd=std(diff(kins.smooth.pl));
	end
	[kins.smooth.max.value,kins.smooth.max.ind]=max(kins.smooth.pl);
	kins.smooth.pl=kins.smooth.pl./kins.smooth.max.value;
	ind_t0=find(kins.smooth.pl(1:kins.smooth.max.ind)>=(opts.maxfraction),1,'first');
	t0=t(ind_t0);
	ind_tsig=find(kins.smooth.pl(1:kins.smooth.max.ind)<=exp(-(opts.sigma^2)./2),1,'last');
	
	%% Plot (if required)
	if(opts.draw)
		h_ax=get(get(0,'CurrentFigure'),'CurrentAxes');
		if(isempty(h_ax))
			h_ax=axes;
			set(get(h_ax,'Parent'),'NumberTitle','off','Name','trplT0Find');
		end
		hold(h_ax,'all');
		opts.styles.smooth=struct('LineStyle','-','LineWidth',2,'Color',[0.8,0.8,0.8]);
		opts.styles.raw=struct('LineStyle','none','Marker','o','MarkerEdgeColor',[0.3,0.3,0.3],'MarkerSize',4);
		opts.styles.T0=struct('Color',hsv2rgb(0,0.85,0.85),'LineWidth',2,'LineStyle',':');
		opts.styles.T2Sig=struct('Color',hsv2rgb(0.66,0.5,0.9),'LineWidth',2,'LineStyle',':');
		plot(h_ax,t,kins.smooth.pl*kins.smooth.max.value*kins.raw.max.value,'LineStyle',opts.styles.smooth.LineStyle,'LineWidth',opts.styles.smooth.LineWidth,'Color',opts.styles.smooth.Color);
		plot(h_ax,t,kins.raw.pl*kins.raw.max.value,'LineStyle',opts.styles.raw.LineStyle,'Marker',opts.styles.raw.Marker,'MarkerEdgeColor',opts.styles.raw.MarkerEdgeColor,'MarkerSize',opts.styles.raw.MarkerSize);
		line('Tag','T0Vert','XData',[t(ind_t0),t(ind_t0)],'YData',[0,max(kins.raw.pl*kins.raw.max.value)],'Color',opts.styles.T0.Color,'LineWidth',opts.styles.T0.LineWidth,'LineStyle',opts.styles.T0.LineStyle);
		line('Tag','T2SigVert','XData',[t(ind_tsig),t(ind_tsig)],'YData',[0,max(kins.raw.pl*kins.raw.max.value)],'Color',opts.styles.T2Sig.Color,'LineWidth',opts.styles.T2Sig.LineWidth,'LineStyle',opts.styles.T2Sig.LineStyle);	
		line('Tag','T0Horiz','XData',[t(1),t(end)],'YData',[opts.maxfraction*kins.smooth.max.value*kins.raw.max.value,opts.maxfraction*kins.smooth.max.value*kins.raw.max.value],'Color',opts.styles.T0.Color,'LineWidth',opts.styles.T0.LineWidth,'LineStyle',opts.styles.T0.LineStyle);
		legend(h_ax,sprintf('Filtered (St. Dev. Fraction: %.4g)',opts.stdevfraction),'Raw',sprintf('T_0: %.4g (index: %d)',t0,ind_t0),sprintf('T_{%.3g\\sigma}: %.4g (index: %d)',opts.sigma,t(ind_tsig),ind_tsig));
		
	end
	
	
	
	%%-- Inline helper functions --%%
	
	%% Set logical option %%
	function ret=setOptionLogical(opt)
		if(isscalar(varg.values{ind_opts}) && islogical(varg.values{ind_opts}))
			opts.(opt)=varg.values{ind_opts};
			ret=true;
		else
			ret=false;
		end
	end

	%% Set numerical option %%
	function ret=setOptionFloat(opt,lims)
		if(isscalar(varg.values{ind_opts}) && isnumeric(varg.values{ind_opts}) && varg.values{ind_opts}<=lims(2) && varg.values{ind_opts}>lims(1))
			opts.(opt)=varg.values{ind_opts};
			ret=true;
		else
			ret=false;
		end
	end
	
	
end
