%% Name:		trplUpdate.m
%% Description:	Update trpl components
%% Authors:		M.J.P Alcocer [marcelo.alcocer@iit.it || marcelo.j.p.alcocer@gmail.com]
%% Licensing:	Copyright  2014  M.J.P Alcocer
%%				This file is part of tprl. trpl is free software: you can redistribute it and/or modify it under the terms of the GNU
%%				General Public License as published by the Free Software Foundation, either version 3 of the License, or any later version.
%%				trpl is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
%%				MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. You should have
%%				received a copy of the GNU General Public License along with trpl. If not, see <http://www.gnu.org/licenses/>.

function files = trplUpdate(varargin)

	%% Syntax:
	%%
	%%		trplUpdate()
	%%		trplUpdate('type')
	%%
	%% Description:
	%%
	%%		trplUpdate() updates the current trpl installation with the latest stable release from the online repository
	%%		(https://bitbucket.org/trpl/trpl.bitbucket.org). Upon confirmation, all local trpl files located in the trpl base directory are
	%%		overwritten. Non-trpl related files are left untouched.
	%%
	%%			Arguments:
	%%				None
	%%
	%%			Returns:
	%%				files:		A cell array containing the names of files updated
	%%
	%%		
	%%		trplUpdate('type') updates the current trpl installation with the latest revision of type 'type'. Valid values for type are currently
	%%		'stable' or 'beta'. trplUpdate('stable') and trplUpdate() are synonymous. True to their nature, beta revisions may be unstable, so users
	%%		are advised to proceed with caution.
	%%
	%%			Arguments:
	%%				'type':		String designating the update revision type ('stable'|'beta')
	%%
	%%			Returns:
	%%				files:		A cell array containing the names of files updated
	%%
	
	
	%% Default return
	files={};																																%#ok<NASGU>
	
	%% Define locations
	basepath=fileparts(which('trplVer'));
	if(isempty(basepath))
		throw(MException('trpl:Update:BasePath','Could not locate the local trpl base directory. Please ensure it is on your Matlab search path'));
	end
	repo.domain='https://bitbucket.org';
	repo.user='trpl';
	repo.name='trpl.bitbucket.org';
	
	%% Determine update type and revision
	if(isempty(varargin))
		varargin{1}='stable';
	end
	switch(lower(varargin{1}))
		case 'stable'
			
			%% Determine latest stable release
			downloads.url=sprintf('%s/%s/%s/downloads',repo.domain,repo.user,repo.name);
			[downloads.html,downloads.status]=urlread(downloads.url);
			if(~downloads.status)
				throw(MException('trpl:Update:HtmlRead','Could not read HTML content at %s. This may be because no network connection is present or because of Bitbucket server downtime. Please check your internet connection and try again',downloads.url));
			end
			repo.rev=regexp(downloads.html,strrep(sprintf('%s/%s/get/(v\\d.\\d.\\d).zip',repo.user,repo.name),'.','\.'),'tokens');
			repo.rev=repo.rev{1}{1};													% Tags always in chronological order?
			
			%% Set confirm string
			cfmstr=sprintf('All current trpl files will be overwritten with the latest stable release (%s).',repo.rev);
			
		case 'beta'			
			
			%% Set latest beta revision
			repo.rev='default';
			
			%% Set confirm string
			cfmstr='All current trpl files will be overwritten with the latest beta revision. True to its nature, this beta revision may be unstable.';
			
		otherwise
			throw(MException('trpl:Update:BadOpt','Option not recognised. Expected: "stable" or "beta". Received: %s',varargin{1}));
	end
	
	%% Pull and unzip desired revision from repository
	zip.url=sprintf('%s/%s/%s/get/%s.zip',repo.domain,repo.user,repo.name,repo.rev);
	fprintf('Downloading files from repository (%s). This may take some time...\n',zip.url);
	zip.files=unzip(zip.url,basepath);
	if(isempty(zip.files))
		throw(MException('trpl:Update:ZipRead','Could not access the files at %s. This may be because no network connection is present or because of Bitbucket server downtime. Please check your internet connection and try again',zip.url));
	end
	fprintf('\bDone - %d files downloaded and extracted.\n',length(zip.files));
	
	%% Determine revision hash
	zip.hash=regexp(zip.files{1},strrep(sprintf('%s-%s-(\\w{12})',repo.user,repo.name),'.','\.'),'tokens');
	zip.hash=zip.hash{1}{1};
	zip.path=fullfile(basepath,sprintf('%s-%s-%s',repo.user,repo.name,zip.hash));

	%% Copy files (upon user confirmation)
% 	if(strcmpi('yes',questdlg(sprintf('All current trpl files will be overwritten with the latest beta revision.\n\nCurrent trpl directory:\n%s\n\nBeta revision:\n%s\n\nTrue to its nature, this beta revision may be unstable.\nDo you wish to proceed?\n',basepath,zip.hash),'Overwrite trpl files?', 'No')))
	if(strcmpi('y',input(sprintf('%s\nCurrent trpl directory:\t%s\nLatest revision:\t\t%s\n\nDo you wish to proceed (y/N)? ',cfmstr,strrep(basepath,'\','\\'),zip.hash),'s')));
		if(~copyfile(fullfile(zip.path,'*'),basepath))
			warning('trpl:Update:Copy','Could not copy revision files to base directory. Source directory: %s. Destination directory: %s',zip.path,basepath);
		end
	else
		fprintf('Update aborted.\n');
	end

	%% Clean up
	if(exist(zip.path,'dir')==7)
		rmdir(zip.path,'s');
	end
	
	%% Return
	files=cellfun(@trplUpdate_buildRet,zip.files,'UniformOutput',false);
	
	
	
	
	%%-- Inline helper functions --%%
	
	%% Build updated file paths
	function newpath = trplUpdate_buildRet(cellPath)
		[~,cell.name,cell.ext]=fileparts(cellPath);
		newpath=fullfile(basepath,[cell.name,cell.ext]);
	end

end

